@@ -79,55 +79,57 @@ struct expr {
 	} v;
 };
 
+AUDIT_HIDDEN_START
+
 /* Free EXPR and all its subexpressions. */
-void expr_free(struct expr *expr) hidden;
+void expr_free(struct expr *expr);
 
 /* Parse STRING.
    On success, return the parsed expression tree.
    On error, set *ERROR to an error string (for free()) or NULL, and return
    NULL.  (*ERROR == NULL is allowed to handle out-of-memory errors) */
-struct expr *expr_parse(const char *string, char **error) hidden;
+struct expr *expr_parse(const char *string, char **error);
 
 /* Create a comparison-expression for FIELD, OP and VALUE.
    On success, return the created expression.
    On error, set errno and return NULL. */
 struct expr *expr_create_comparison(const char *field, unsigned op,
-				    const char *value) hidden;
+				    const char *value);
 
 /* Create a timestamp comparison-expression for with OP, SEC, MILLI.
    On success, return the created expression.
    On error, set errno and return NULL. */
 struct expr *expr_create_timestamp_comparison(unsigned op, time_t sec,
-					      unsigned milli) hidden;
+					      unsigned milli);
 
 /* Create an extended timestamp comparison-expression for with OP, SEC, 
    MILLI, and SERIAL.
    On success, return the created expression.
    On error, set errno and return NULL. */
 struct expr *expr_create_timestamp_comparison_ex(unsigned op, time_t sec,
-				      unsigned milli, unsigned serial) hidden;
+				      unsigned milli, unsigned serial);
 
 /* Create an EO_FIELD_EXISTS-expression for FIELD.
    On success, return the created expression.
    On error, set errno and return NULL. */
-struct expr *expr_create_field_exists(const char *field) hidden;
+struct expr *expr_create_field_exists(const char *field);
 
 /* Create a \regexp expression for regexp comparison.
    On success, return the created expression.
    On error, set errno and return NULL. */
-struct expr *expr_create_regexp_expression(const char *regexp) hidden;
+struct expr *expr_create_regexp_expression(const char *regexp);
 
 /* Create a binary expresion for OP and subexpressions E1 and E2.
    On success, return the created expresion.
    On error, set errno and return NULL. */
-struct expr *expr_create_binary(unsigned op, struct expr *e1, struct expr *e2)
-	hidden;
+struct expr *expr_create_binary(unsigned op, struct expr *e1, struct expr *e2);
 
 /* Evaluate EXPR on RECORD in AU->le.
    Return 1 if EXPR is true, 0 if it false or if it fails.
    (No error reporting facility is provided; an invalid term is considered to
    be false; e.g. !invalid is true.) */
-int expr_eval(auparse_state_t *au, rnode *record, const struct expr *expr)
-	hidden;
+int expr_eval(auparse_state_t *au, rnode *record, const struct expr *expr);
+
+AUDIT_HIDDEN_END
 
 #endif
