@@ -178,6 +178,8 @@ static int extract_type(const char *str)
 	const char *tptr, *ptr2, *ptr = str;
 	if (*str == 'n') {
 		ptr = strchr(str+1, ' ');
+		if (ptr == NULL)
+			return -1; // Malformed - bomb out
 		ptr++;
 	}
 	// ptr should be at 't'
@@ -186,6 +188,8 @@ static int extract_type(const char *str)
 	tptr = strndupa(ptr, ptr2 - ptr);
 	// find =
 	str = strchr(tptr, '=');
+	if (str == NULL)
+		return -1; // Malformed - bomb out
 	// name is 1 past
 	str++;
 	return audit_name_to_msg_type(str);
@@ -193,32 +197,35 @@ static int extract_type(const char *str)
 
 void distribute_event(struct auditd_event *e)
 {
-	int attempt = 0, route = 1;;
+	int attempt = 0, route = 1;
 
-	// If a network event, type is 0
+	/* If type is 0, then its a network originating event */
 	if (e->reply.type == 0) {
+		// See if we are distributing network originating events
 		if (!dispatch_network_events())
 			route = 0;
-		else // We only need the type if its being routed
+		else	// We only need the original type if its being routed
 			e->reply.type = extract_type(e->reply.message);
-	} else {
-		// Do the formatting
-	}
+	} else if (e->reply.type != AUDIT_DAEMON_RECONFIG)
+		// All other events need formatting
+		format_event(e);
+	else
+		route = 0; // Don't DAEMON_RECONFIG events until after enqueue
 
 	/* Make first attempt to send to plugins */
 	if (route && dispatch_event(&e->reply, attempt) == 1)
 		attempt++; /* Failed sending, retry after writing to disk */
 
 	/* End of Event is for realtime interface - skip local logging of it */
-	if (e->reply.type != AUDIT_EOE) {
-		/* Write to local disk */
-		enqueue_event(e);
-	} else
-		free(e);	// This function takes custody of the memory
+	if (e->reply.type != AUDIT_EOE)
+		handle_event(e); /* Write to local disk */
 
-	/* Last chance to send...maybe the pipe is empty now. *
-	if (attempt && route) 
-		dispatch_event(&e->reply, attempt); */
+	/* Last chance to send...maybe the pipe is empty now. */
+	if ((attempt && route) || (e->reply.type == AUDIT_DAEMON_RECONFIG))
+		dispatch_event(&e->reply, attempt);
+
+	/* Free msg and event memory */
+	cleanup_event(e);
 }
 
 /*
@@ -238,26 +245,23 @@ int send_audit_event(int type, const char *str)
 	}
 
 	e->reply.type = type;
-	e->reply.message = (char *)malloc(DMSG_SIZE);
-	if (e->reply.message == NULL) {
-		free(e);
-		audit_msg(LOG_ERR, "Cannot allocate local event message");
-		return 1;
-	}
 	if (seq_num == 0) {
 		srand(time(NULL));
 		seq_num = rand()%10000;
 	} else
 		seq_num++;
+	// Write event into netlink area like normal events
 	if (gettimeofday(&tv, NULL) == 0) {
-		e->reply.len = snprintf((char *)e->reply.message,
+		e->reply.len = snprintf((char *)e->reply.msg.data,
 			DMSG_SIZE, "audit(%lu.%03u:%u): %s", 
 			tv.tv_sec, (unsigned)(tv.tv_usec/1000), seq_num, str);
 	} else {
-		e->reply.len = snprintf((char *)e->reply.message,
+		e->reply.len = snprintf((char *)e->reply.msg.data,
 			DMSG_SIZE, "audit(%lu.%03u:%u): %s", 
 			(unsigned long)time(NULL), 0, seq_num, str);
 	}
+	// Point message at the netlink buffer like normal events
+	e->reply.message = e->reply.msg.data;
 	if (e->reply.len > DMSG_SIZE)
 		e->reply.len = DMSG_SIZE;
 
@@ -431,6 +435,7 @@ static void netlink_handler(struct ev_loop *loop, struct ev_io *io,
 			shutdown_dispatcher();
 			return;
 		}
+		cur_event->ack_func = NULL;
 	}
 	if (audit_get_reply(fd, &cur_event->reply, 
 			    GET_REPLY_NONBLOCKING, 0) > 0) {
@@ -591,9 +596,10 @@ int main(int argc, char *argv[])
 	}
 
 #ifndef DEBUG
-	/* Make sure we are root */
-	if (getuid() != 0) {
-		fprintf(stderr, "You must be root to run this program.\n");
+	/* Make sure we can do our job */
+	if (!audit_can_control() || !audit_can_read()) {
+		fprintf(stderr,
+		"You must be root or have capabilities to run this program.\n");
 		return 4;
 	}
 #endif
@@ -616,8 +622,10 @@ int main(int argc, char *argv[])
 	setrlimit(RLIMIT_CPU, &limit);
 
 	/* Load the Configuration File */
-	if (load_config(&config, TEST_AUDITD))
+	if (load_config(&config, TEST_AUDITD)) {
+		free_config(&config);
 		return 6;
+	}
 
 	// This can only be set at start up
 	opt_aggregate_only = !config.local_events;
@@ -628,6 +636,7 @@ int main(int argc, char *argv[])
 		if (rc == -1 && errno) {
 			audit_msg(LOG_ERR, "Cannot change priority (%s)", 
 					strerror(errno));
+			free_config(&config);
 			return 1;
 		}
 	} 
@@ -638,6 +647,7 @@ int main(int argc, char *argv[])
 			audit_msg(LOG_ERR, "Cannot daemonize (%s)",
 				strerror(errno));
 			tell_parent(FAILURE);
+			free_config(&config);
 			return 1;
 		} 
 		openlog("auditd", LOG_PID, LOG_DAEMON);
@@ -647,6 +657,7 @@ int main(int argc, char *argv[])
 	if ((fd = audit_open()) < 0) {
         	audit_msg(LOG_ERR, "Cannot open netlink audit socket");
 		tell_parent(FAILURE);
+		free_config(&config);
 		return 1;
 	}
 
@@ -656,6 +667,7 @@ int main(int argc, char *argv[])
 		if (pidfile)
 			unlink(pidfile);
 		tell_parent(FAILURE);
+		free_config(&config);
 		return 1;
 	}
 
@@ -663,6 +675,7 @@ int main(int argc, char *argv[])
 		if (pidfile)
 			unlink(pidfile);
 		tell_parent(FAILURE);
+		free_config(&config);
 		return 1;
 	}
 
@@ -671,6 +684,7 @@ int main(int argc, char *argv[])
 		if (pidfile)
 			unlink(pidfile);
 		tell_parent(FAILURE);
+		free_config(&config);
 		return 1;
 	}
 
@@ -679,6 +693,7 @@ int main(int argc, char *argv[])
 		if (pidfile)
 			unlink(pidfile);
 		tell_parent(FAILURE);
+		free_config(&config);
 		return 1;
 	}
 	fcntl(pipefds[0], F_SETFD, FD_CLOEXEC);
@@ -696,6 +711,7 @@ int main(int argc, char *argv[])
 				unlink(pidfile);
 			tell_parent(FAILURE);
 			close_pipes();
+			free_config(&config);
 			return 1;
 		}
 		if (getsubj(subj))
@@ -717,6 +733,7 @@ int main(int argc, char *argv[])
 			shutdown_dispatcher();
 			tell_parent(FAILURE);
 			close_pipes();
+			free_config(&config);
 			return 1;
 		}
 	}
@@ -750,6 +767,7 @@ int main(int argc, char *argv[])
 		shutdown_dispatcher();
 		tell_parent(FAILURE);
 		close_pipes();
+		free_config(&config);
 		return 1;
 	}
 
@@ -773,6 +791,7 @@ int main(int argc, char *argv[])
 		shutdown_dispatcher();
 		tell_parent(FAILURE);
 		close_pipes();
+		free_config(&config);
 		return 1;
 	}
 
